/* GKzeta - Zetagrid plugin for GKrellM
|  Copyright (C) 2003 - Viljem Skornik
|
|  Author:  Viljem Skornik    <bluesman at byteme dot no>
|  Latest versions might be found at:
|		http://bluesman.byteme.no/index.php?page=projects
|
|  This program is free software which I release under the GNU General Public
|  License. You may redistribute and/or modify this program under the terms
|  of that license as published by the Free Software Foundation; either
|  version 2 of the License, or (at your option) any later version.
|
|  This program is distributed in the hope that it will be useful,
|  but WITHOUT ANY WARRANTY; without even the implied warranty of
|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|  GNU General Public License for more details.
|
|  To get a copy of the GNU General Puplic License,  write to the
|  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/************************************************************************/
/*  DO NOT EDIT BELOW THIS LINE!!!									    */
/************************************************************************/

#include <time.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "gkzeta.h"

/*
|  Other settings.
*/
/* Define the FULL scale of the chart. */
#define CHART_FULL_SCALE 10000
/* Define the FULL scale of the krell (that little horizontal statusbar). */
#define KRELL_FULL_SCALE 10000
/* Define plugin placement. */
#define PLUGIN_PLACEMENT (MON_MAIL | MON_INSERT_AFTER)
#define CONFIG_NAME "Zetagrid"
#define STYLE_NAME "zetagrid"

static GkrellmMonitor *monitor;
static GkrellmChart *chart;
static GkrellmChartdata *chart_data;
static GkrellmChartconfig *chart_config;
static gint style_id;
static GkrellmKrell *krell;

static gint ZetaClientPID = 0;
gdouble total_time[2];
gboolean my_switch;
gdouble seconds[2];

typedef struct {
	gdouble Performance;
	long long Progress;
	time_t CurTimePos;
	time_t FirstTimePos;
	long long CurPos;
	long long FirstPos;
}  ZetaDataStruct;

ZetaDataStruct Zeta;

/*
|	Get's the progress out of the TMP_FILE.
|	How to do it, was learned from the java client.
*/
long long getProgress() {
	long long progress[2];
	gchar buffer[1024];
	gchar *b, *s;
	FILE *fp;
	struct stat modinfo;

	// get last modification info on the zeta_zeros.tmp and info.tmp files
	if (!stat(TMP_FILE,&modinfo))
		Zeta.CurTimePos = modinfo.st_mtime;
	else
		Zeta.CurTimePos = 0;
	if (!stat(INFO_TMP_FILE,&modinfo))
		Zeta.FirstTimePos = modinfo.st_mtime;
	else
		Zeta.FirstTimePos = 0;

	fp = fopen(TMP_FILE, "r");
	if (fp == NULL) {
		if (DEBUG == 1) {
			printf("Error: Cannot open file %s.\n", TMP_FILE);
		}
		Zeta.Progress = 0;
		Zeta.Performance = 0;
		return 0;
	} else {
		fgets (buffer, sizeof(buffer), fp);
		fclose(fp);
		b = buffer;
		s = strsep(&b, ","); progress[0] = atof(s);
		s = strsep(&b, ","); progress[1] = atof(s);
		s = strsep(&b, ","); progress[2] = atof(s);

		// store the progress in our Zeta structure
		Zeta.Progress = (progress[0] - progress[1])*10000/progress[2];

		// get position information
		Zeta.FirstPos = progress[1];
		Zeta.CurPos = progress[0];

		if (Zeta.FirstTimePos != 0) {
			Zeta.Performance = (double)(Zeta.CurPos - Zeta.FirstPos) / (double)(Zeta.CurTimePos - Zeta.FirstTimePos);
			//Zeta.Performance = Zeta.Performance / 1000;
		}

		return ((progress[0] - progress[1])*10000/progress[2]);
	}
}

/*
|	Get's the PID of the ZetaGrid client process.
*/
gint getZetaClientPID() {
	gchar buffer[10];
	FILE *fp;

	if ((fp = popen("ps arx | grep zeta_client.jar | grep -v 'grep' | tail -1 | awk '{printf($1)}'", "r")) != NULL) {
		fgets(buffer, sizeof(buffer), fp);
		pclose(fp);
		if (DEBUG == 1) {
			printf("Info: ZetaGrid client PID = %s, %d\n", buffer, atoi(buffer));
		}
		return atoi(buffer);
	} else {
		if (DEBUG == 1) {
			printf("Error: Cannot get the PID of the ZetaGrid client.\n");
		}
		return 0;
	}
}

/*
|	Draws the chart.
*/
static void draw_chart(void) {
	gdouble CPU;
	gchar usage[31];

	gkrellm_draw_chartdata(chart);
	//gkrellm_draw_chart_text(chart, style_id, "\\f\\D0\\cZetagrid");
	CPU = getProgress()/100.0;
	sprintf(usage, "\\fcomp\\r%0.1lf\\n\\fperf\\r%0.1lf", CPU, Zeta.Performance);
	if (DEBUG == 1) {
		printf("CPU usage = %s\n", usage);
	}
	gkrellm_store_chartdata(chart, 0, (int) (CPU*100));
	gkrellm_draw_chart_text(chart, style_id, usage);
	gkrellm_draw_chart_to_screen(chart);
}


/*
|	Draws the krell (the progress indicator).
*/
static void draw_krell(void) {
	gkrellm_update_krell(chart->panel, krell, getProgress());
	gkrellm_draw_panel_layers(chart->panel);
}

/*
|	Updates plugin values accordingly.
*/
static void update_plugin() {
	if (GK.second_tick) {
		if (DEBUG == 1) {
			ZetaClientPID = getZetaClientPID();
		}
		draw_chart();
	}
	if (GK.minute_tick) {
	//	ZetaClientPID = getZetaClientPID();
		draw_krell();
	}
}

static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *ev) {
	if (widget == chart->panel->drawing_area) {
		gdk_draw_pixmap(widget->window,
						widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
						chart->panel->pixmap,
						ev->area.x, ev->area.y,	ev->area.x,	ev->area.y,
						ev->area.width, ev->area.height);
	}
	return FALSE;
}

static gint chart_expose_event(GtkWidget *widget, GdkEventExpose * ev) {
	if (widget == chart->drawing_area) {
		gdk_draw_pixmap(widget->window,
						widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
						chart->pixmap,
						ev->area.x, ev->area.y, ev->area.x, ev->area.y,
						ev->area.width, ev->area.height);
	}
	return FALSE;
}

static void create_plugin(GtkWidget *vbox, gint first_create) {
	GkrellmStyle *style;
	GkrellmPiximage *krell_image;
	GkrellmPanel *p;

	if (first_create) {
		chart = gkrellm_chart_new0();
		p = gkrellm_panel_new0();
		chart->panel = p;
	}
	memset(&Zeta,0,sizeof(ZetaDataStruct));

	gkrellm_set_chart_height_default(chart, CHART_HEIGHT);
	gkrellm_chart_create(vbox, monitor, chart, &chart_config);
	chart_data = gkrellm_add_default_chartdata(chart, "CPU usage");
	gkrellm_monotonic_chartdata(chart_data, FALSE);
	//gkrellm_set_chartdata_draw_style_default(chart_data, CHARTDATA_IMPULSE);
	gkrellm_set_chartdata_draw_style_default(chart_data, CHARTDATA_LINE);
	gkrellm_set_chartdata_flags(chart_data, CHARTDATA_ALLOW_HIDE);
	gkrellm_set_draw_chart_function(chart, draw_chart, NULL);
	gkrellm_chartconfig_grid_resolution_adjustment(chart_config, TRUE,
												   0, AUTO_RESIZE,
												   (gfloat) CHART_FULL_SCALE,
												   0, 0, 0, 70);

	style = gkrellm_meter_style(style_id);

	krell_image = gkrellm_krell_meter_piximage(style_id);
	krell = gkrellm_create_krell(chart->panel, krell_image, style);
	gkrellm_monotonic_krell_values(krell, FALSE);
	gkrellm_set_krell_full_scale(krell, KRELL_FULL_SCALE, 1);

	gkrellm_panel_configure(chart->panel, "Zetagrid", style);
	gkrellm_panel_create(vbox, monitor, chart->panel);

	gkrellm_alloc_chartdata(chart);

	if (first_create) {
		gtk_signal_connect(GTK_OBJECT(chart->panel->drawing_area),
						   "expose_event", (GtkSignalFunc) panel_expose_event,
						   NULL);
		gtk_signal_connect(GTK_OBJECT(chart->drawing_area),
						   "expose_event", (GtkSignalFunc) chart_expose_event,
						   NULL);
		ZetaClientPID = getZetaClientPID();
		if (DEBUG == 1) {
			printf("Info ZetaGrid client PID = %d\n", ZetaClientPID);
		}
	}

	gkrellm_draw_chart_text(chart, style_id, "Zetagrid");
	gkrellm_draw_panel_layers(chart->panel);
	draw_krell();
	draw_chart();
}

static GkrellmMonitor plugin_mon = {
	"Zetagrid",
	0,
	create_plugin,
	update_plugin,
	NULL,
	NULL,
	NULL,
	NULL,
	"ZetagridA",
	NULL,
	NULL,
	NULL,
	PLUGIN_PLACEMENT,
	NULL,
	NULL,
};

GkrellmMonitor *gkrellm_init_plugin(void) {
	style_id = gkrellm_add_chart_style(&plugin_mon, STYLE_NAME);
	monitor = &plugin_mon;
	return &plugin_mon;
}
